import os
import subprocess


def srai_killall():
    if os.name == "nt":
        # windows
        subprocess.run("taskkill /F /IM python.exe", shell=True)
    else:
        # linux
        subprocess.run("killall python", shell=True)


def main():
    srai_killall()


if __name__ == "__main__":
    main()



from abc import ABC, abstractmethod
from typing import List


class DirStoreBase(ABC):

    def __init__(self) -> None:
        pass

    # def copy_to(self, store_to: "DirStoreBase") -> None:
    #     dict_dir = self.load_dir_all()
    #     for dir_id, dir in dict_dir.items():
    #         store_to.save_dir(dir_id, dir)

    @abstractmethod
    def exists_dir(self, dir_id: str) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def count_dir(self) -> int:
        raise NotImplementedError()

    @abstractmethod
    def load_dir(self, dir_id: str, path_dir_target: str) -> None:
        raise NotImplementedError()

    def try_load_dir(self, dir_id: str, path_dir_target: str) -> None:
        if self.exists_dir(dir_id):
            return self.load_dir(dir_id, path_dir_target)
        else:
            return None

    @abstractmethod
    def save_dir(self, dir_id: str, path_dir_source: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def load_list_dir_id(self) -> List[str]:
        raise NotImplementedError()

    @abstractmethod
    def delete_dir(self, dir_id: str) -> None:
        raise NotImplementedError()

    def delete_dir_all(self) -> int:
        list_dir_id = self.load_list_dir_id()
        for dir_id in list_dir_id:
            self.delete_dir(dir_id)
        return len(list_dir_id)



import hashlib
from typing import List

from pydantic import BaseModel


class Encoding(BaseModel):
    encoding_id: str
    instance_id: str
    encoder_id: str
    created_at_posix_timestamp: int
    list_metadata: List[dict]
    list_encoding: List[List[float]]

    @staticmethod
    def create_encoding_id(instance_id: str, encoder_id: str):
        return hashlib.sha256(f"{instance_id}-{encoder_id}".encode()).hexdigest()



from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.bytes_store_memory import BytesStoreMemory
from srai_core.store.database_base import DatabaseBase
from srai_core.store.document_store_base import DocumentStoreBase
from srai_core.store.document_store_memory import DocumentStoreMemory


class DatabaseMemory(DatabaseBase):

    def __init__(self, database_name: str):
        super().__init__(database_name)

    def get_document_store(self, name: str) -> DocumentStoreBase:
        return DocumentStoreMemory()

    def get_bytes_store(self, collection_name: str) -> BytesStoreBase:
        return BytesStoreMemory()



import asyncio

from srai_core.command_handler_base import CommandHandlerBase
from srai_core.command_handler_subprocess import CommandHandlerSubprocess


async def srai_release_code_public(command_handler: CommandHandlerBase):
    print("Delete old distribution")
    command = "rm -rf dist"
    command_handler.execute(command)

    print("Creating distribution")
    command = "poetry build"
    command_handler.execute(command)

    print("Uploading distribution")
    command = "twine upload dist/*"
    command_handler.execute(command)


if __name__ == "__main__":
    command_handler = CommandHandlerSubprocess()
    asyncio.run(srai_release_code_public(command_handler))



import asyncio
import json
import sys

from srai_core.command_handler_base import CommandHandlerBase
from srai_core.tools_docker import get_image_tag, list_container_name, remove_container, stop_container


async def srai_docker_stop(deployment: dict):
    for deploy_target in deployment["list_deploy_target"]:
        command_handler = CommandHandlerBase.from_dict(deploy_target["command_handler"])

        image_tag = get_image_tag()

        container_name = image_tag.split(":")[0].split("/")[-1]

        list_container_name_present = list_container_name(command_handler)
        if container_name in list_container_name_present:
            stop_container(command_handler, container_name)
            remove_container(command_handler, container_name)


if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: srai_docker_stop deployment_file.json")
        sys.exit(1)

    path_deployment_file = sys.argv[1]
    with open(path_deployment_file) as f:
        deployment = json.load(f)
    asyncio.run(srai_docker_stop(deployment))



from typing import Dict, Generic, List, Optional, Tuple, Type, TypeVar

from pydantic import BaseModel

from srai_core.store.document_store_base import DocumentStoreBase

T = TypeVar("T", bound=BaseModel)


class ObjectStore(Generic[T]):

    def __init__(self, document_store: DocumentStoreBase, model_class: Type[T]):
        self.document_store = document_store
        self.model_class = model_class

    def _document_to_object(self, document: dict) -> T:
        return self.model_class(**document)  # type: ignore

    def copy_to(self, store_to: "DocumentStoreBase") -> None:
        self.document_store.copy_to(store_to)

    def exists_object(self, document_id: str) -> bool:
        return self.document_store.exists_document(document_id)

    def count_object(self) -> int:
        return self.document_store.count_document()

    def load_object(self, document_id: str) -> T:
        document = self.document_store.load_document(document_id)
        return self._document_to_object(document)

    def try_load_object(self, bytes_id: str) -> Optional[T]:
        if self.document_store.exists_document(bytes_id):
            return self.load_object(bytes_id)
        else:
            return None

    def load_object_all(self) -> Dict[str, T]:
        dict_document = self.document_store.load_document_all()
        dict_object = {}
        for document_id, document in dict_document.items():
            dict_object[document_id] = self._document_to_object(document)
        return dict_object

    def load_list_object_id(self) -> List[str]:
        return self.document_store.load_list_document_id()

    def load_object_dict_for_query(
        self, query: Dict[str, str], order_by: List[Tuple[str, bool]], limit: int = 0, offset: int = 0
    ) -> Dict[str, T]:
        dict_document = self.document_store.load_document_dict_for_query(query, order_by, limit, offset)
        dict_object = {}
        for document_id, document in dict_document.items():
            dict_object[document_id] = self._document_to_object(document)
        return dict_object

    def load_object_for_query(self, query: Dict[str, str]) -> Optional[T]:
        document = self.document_store.load_document_for_query(query)
        if document is not None:
            return self._document_to_object(document)
        else:
            return None

    def delete_object(self, object_id: str) -> None:
        self.document_store.delete_document(object_id)

    def delete_object_all(self) -> int:
        list_object_id = self.load_list_object_id()
        for object_id in list_object_id:
            self.delete_object(object_id)
        return len(list_object_id)

    def save_object(self, object_id: str, object: T) -> None:
        document = object.model_dump()
        self.document_store.save_document(object_id, document)



import base64
from typing import Dict, List

from srai_core.model.docker_registry_base import DockerRegistryBase
from srai_core.tools_env import get_client_ecr


class DockerRegistryEcr(DockerRegistryBase):
    def __init__(self, account_id: str, region_name: str):
        self.client_ecr = get_client_ecr()
        registry_url = DockerRegistryEcr.get_registry_url(account_id, region_name)
        token = DockerRegistryEcr.get_ecr_login_token()
        username, password = token.split(":")
        super().__init__(registry_url, username, password)
        self.account_id = account_id
        self.region_name = region_name

    @staticmethod
    def get_ecr_login_token():
        ecr_client = get_client_ecr()
        response = ecr_client.get_authorization_token()
        token = response["authorizationData"][0]["authorizationToken"]
        return base64.b64decode(token).decode("utf-8")

    @staticmethod
    def get_registry_url(account_id, region_name) -> str:
        return f"{account_id}.dkr.ecr.{region_name}.amazonaws.com"

    def image_list(self) -> Dict[str, List[str]]:

        # Create an ECR client
        client_ecr = get_client_ecr()

        # Get a list of all repositories in your ECR
        repositories = client_ecr.describe_repositories()

        dict_list_images = {}
        for repo in repositories["repositories"]:
            repo_name = repo["repositoryName"]
            dict_list_images[repo_name] = []

            # Paginate through image details since ECR might have many images
            paginator = client_ecr.get_paginator("list_images")
            for page in paginator.paginate(repositoryName=repo_name):
                for image in page["imageIds"]:
                    image_tag = image.get("imageTag", "latest")
                    dict_list_images[repo_name].append(image_tag)
            dict_list_images[repo_name].sort()  # TODO use version compare
        return dict_list_images

    def repository_create(self, repository_name: str, is_tag_immutable: bool = True):
        if is_tag_immutable:
            image_tag_mutability = "IMMUTABLE"
        else:
            image_tag_mutability = "MUTABLE"
        self.client_ecr.create_repository(repositoryName=repository_name, imageTagMutability=image_tag_mutability)

    def repository_delete(self, repository_name: str) -> None:
        self.client_ecr.delete_repository(repositoryName=repository_name)

    def repository_exists(self, repository_name: str):
        list_repository = self.repository_list()
        for repo in list_repository:
            if repo["repositoryName"] == repository_name:
                return True
        return False

    def repository_list(self) -> List[Dict]:
        response = self.client_ecr.describe_repositories()
        list_repository = response["repositories"]
        return list_repository



from typing import Dict, List

from srai_core.store.document_store_base import DocumentStoreBase


class DocumentStoreMemory(DocumentStoreBase):
    def __init__(self):
        self.dict_document = {}

    def save_document(self, document_id: str, document: dict) -> None:
        self.dict_document[document_id] = document

    def load_document(self, document_id: str) -> dict:
        return self.dict_document[document_id]

    def load_list_document_id(self) -> List[str]:
        return list(self.dict_document.keys())

    def delete_document(self, document_id: str) -> None:
        del self.dict_document[document_id]

    def exists_document(self, document_id: str) -> bool:
        return document_id in self.dict_document

    def load_document_all(self) -> Dict[str, dict]:
        return self.dict_document

    def count_document(self) -> int:
        return len(self.dict_document)



import os
from typing import Dict, List

import paramiko
from paramiko import SSHClient
from pip._vendor import tomli  # TODO replace in 3.11

from srai_core.command_handler_base import CommandHandlerBase
from srai_core.model.docker_registry_base import DockerRegistryBase
from srai_core.model.docker_registry_ecr import DockerRegistryEcr
from srai_core.tools_env import get_string_from_env


def get_client_ssh(hostname: str, username: str, path_file_pem: str, port: int = 22) -> SSHClient:
    # Initialize SSH client
    ssh_client = paramiko.SSHClient()
    ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())  # Automatically add the server's host key
    # Load private key
    private_key = paramiko.RSAKey(filename=path_file_pem)

    # Connect using the private key for authentication
    ssh_client.connect(hostname, port, username, pkey=private_key)
    return ssh_client


def read_toml_dict() -> Dict:
    if os.path.isfile("pyproject.toml"):
        with open("pyproject.toml", "rb") as file:
            return tomli.load(file)
    else:
        raise Exception("pyproject.toml not found")


def get_project_name() -> str:
    return read_toml_dict()["tool"]["poetry"]["name"]


def get_project_version() -> str:
    return read_toml_dict()["tool"]["poetry"]["version"]


def get_image_tag() -> str:
    toml = read_toml_dict()
    image_name = "srai/" + toml["tool"]["poetry"]["name"]
    image_version = toml["tool"]["poetry"]["version"]
    image_tag = f"{image_name}:{image_version}"
    return image_tag


# TODO all should be async
async def build_docker(command_handler: CommandHandlerBase, path=None) -> None:
    image_tag = get_image_tag()
    # stop all containers using this image
    container_name = image_tag.split(":")[0].split("/")[-1]
    stop_container(command_handler, container_name)

    # remove all containers using this image
    remove_container(command_handler, container_name)

    # remove the image
    command = f"docker rmi {image_tag}"
    command_handler.execute(command)

    # build the image
    if path is not None:
        command = f"cd {path}; docker build -t {image_tag} {path}"  # TODO add --no-cache
    else:
        command = f"docker build -t {image_tag} ."  # TODO add --no-cache
    command_handler.execute(command)


async def release_docker_local_to_aws(command_handler: CommandHandlerBase) -> None:
    image_tag = get_image_tag()
    account_id = get_string_from_env("AWS_ACCOUNT_ID")
    region_name = get_string_from_env("AWS_REGION_NAME")
    docker_registry = DockerRegistryEcr(account_id, region_name)
    repository_name = image_tag.split(":")[0]
    docker_registry.login(command_handler)
    registry_url = docker_registry.registry_url

    # check if repository exists in registry

    list_repository = docker_registry.repository_list()
    for repo in list_repository:
        print(repo)

    if not docker_registry.repository_exists(repository_name):
        docker_registry.repository_create(repository_name)

    # retag the image
    command = f"docker tag {image_tag} "
    command += f"{registry_url}/{image_tag}"
    command_handler.execute(command)

    # push the image
    command = f"docker push {registry_url}/{image_tag}"
    command_handler.execute(command)


def start_container_command(
    image_tag: str,
    container_name: str,
    dict_env: Dict[str, str],
    dict_port: Dict[str, str],
) -> str:
    command = f"docker run -d --name {container_name}"
    for key, value in dict_env.items():
        command += f" -e {key}={value}"
    for key, value in dict_port.items():
        command += f" -p {key}:{value}"
    command += f" {image_tag}"
    return command


async def start_container(
    command_handler: CommandHandlerBase,
    docker_registry: DockerRegistryBase,
    image_tag: str,
    container_name: str,
    dict_env: Dict[str, str],
    dict_port: Dict[str, str],
) -> None:
    print("here")
    # No username means that the image is local
    if docker_registry.get_username() != "":
        print(f"username: {docker_registry.get_username()}")
        docker_registry.login(command_handler)
        docker_registry.image_pull(command_handler, image_tag)
    command = start_container_command(image_tag, container_name, dict_env, dict_port)

    if type(docker_registry) is DockerRegistryEcr:
        command = command.replace(image_tag, f"{docker_registry.registry_url}/{image_tag}")
    # Execute the command
    command_handler.execute(command)


def list_container_name(command_handler: CommandHandlerBase) -> List[str]:
    # Command to check if the Docker container is running
    command = "docker ps -a --format '{{.Names}}'"
    output, _ = command_handler.execute(command)
    return output.strip().split("\n")


def parse_table(list_header: List[str], str_table: str) -> List[Dict[str, str]]:
    list_start = []
    list_end = []
    list_dict_line = []
    list_line = str_table.split("\n")
    for header in list_header:
        list_start.append(str_table.find(header))

    for i, start in enumerate(list_start[:-1]):
        list_end.append(list_start[i + 1])
    list_end.append(-1)

    for line in list_line[1:]:
        dict_line = {}
        for header, start, end in zip(list_header, list_start, list_end):
            if end == -1:
                dict_line[header] = line[start:].strip()
            else:
                dict_line[header] = line[start:end].strip()
        list_dict_line.append(dict_line)
    return list_dict_line


def list_container_status(
    command_handler: CommandHandlerBase,
) -> List[Dict[str, str]]:
    # Command to check if the Docker container is running
    command = "docker ps -a"

    # Execute the command
    output_out, _ = command_handler.execute(command)
    list_header = [
        "CONTAINER ID",
        "IMAGE",
        "COMMAND",
        "CREATED",
        "STATUS",
        "PORTS",
        "NAMES",
    ]
    return parse_table(list_header, output_out)


def container_logs(
    command_handler: CommandHandlerBase,
    container_name: str,
    logs_count: int,
) -> List[str]:
    # Command to check if the Docker container is running
    command = f"docker logs --tail {logs_count} {container_name}"

    # Execute the command
    output_out, _ = command_handler.execute(command)
    return output_out.split("\n")


def stop_container(
    command_handler: CommandHandlerBase,
    container_name: str,
) -> None:
    # Command to stop the Docker container
    command = f"docker stop {container_name}"

    # Execute the command
    command_handler.execute(command)


def container_status(command_handler: CommandHandlerBase, container_name: str) -> str:
    # Command to check if the Docker container is running
    command = f"docker inspect --format='{{{{.State.Status}}}}' {container_name}"

    # Execute the command
    output_out, _ = command_handler.execute(command)
    return output_out.strip()


def remove_container(
    command_handler: CommandHandlerBase,
    container_name: str,
) -> None:
    # Command to stop the Docker container
    command = f"docker rm {container_name}"

    # Execute the command
    command_handler.execute(command)


def clone_repository(
    command_handler: CommandHandlerBase,
    package_name: str,
    path: str,
    git_token: str,
) -> None:
    # clear the directory
    command = f"rm -rf {path}/{package_name}"
    command_handler.execute(command)
    # clone the repository with a token

    command = f"git clone https://{git_token}@github.com/southriverai/{package_name}.git {path}/{package_name}"
    command_handler.execute(command)



from abc import ABC, abstractmethod
from typing import Dict, List, Optional


class BytesStoreBase(ABC):
    # file store for

    def __init__(self) -> None:
        pass

    @abstractmethod
    def exists_bytes(self, bytes_id: str) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def count_bytes(self) -> int:
        raise NotImplementedError()

    @abstractmethod
    def load_bytes(self, bytes_id: str) -> bytes:
        raise NotImplementedError()

    def try_load_bytes(self, bytes_id: str) -> Optional[bytes]:
        if self.exists_bytes(bytes_id):
            return self.load_bytes(bytes_id)
        else:
            return None

    @abstractmethod
    def load_list_bytes_id(self) -> List[str]:
        raise NotImplementedError()

    @abstractmethod
    def load_bytes_all(self) -> Dict[str, bytes]:
        raise NotImplementedError()

    @abstractmethod
    def delete_bytes(self, bytes_id: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def save_bytes(self, bytes_id: str, bytes: bytes) -> None:
        raise NotImplementedError()



import asyncio
import json
import sys

from srai_core.command_handler_base import CommandHandlerBase
from srai_core.model.docker_registry_base import DockerRegistryBase
from srai_core.model.docker_registry_ecr import DockerRegistryEcr
from srai_core.script.srai_release import srai_release
from srai_core.tools_docker import get_image_tag, remove_container, start_container, stop_container
from srai_core.tools_env import get_deployment, get_string_from_env


async def srai_deploy(deployment: dict):
    await srai_release(deployment)
    for deploy_target in deployment["list_deploy_target"]:
        command_handler = CommandHandlerBase.from_dict(deploy_target["command_handler"])
        deploy_target_type = deploy_target["deploy_target_type"]
        if deploy_target_type == "deploy_docker":
            image_tag = get_image_tag()
            dict_env = deploy_target["environment_dict"]
            if "port_dict" in deploy_target:
                dict_port = deploy_target["port_dict"]
            else:
                dict_port = {}
            dict_env["IMAGE_TAG"] = image_tag

            container_name = image_tag.split(":")[0].split("/")[-1]
            registry = DockerRegistryBase("", "kozzion", "wrc0ceq5xjk-ext!PET")
            stop_container(command_handler, container_name)
            remove_container(command_handler, container_name)

            await start_container(
                command_handler,
                registry,
                image_tag,
                container_name,
                dict_env,
                dict_port,
            )
        elif deploy_target_type == "deploy_docker_aws":
            image_tag = get_image_tag()
            dict_env = deploy_target["environment_dict"]
            if "port_dict" in deploy_target:
                dict_port = deploy_target["port_dict"]
            else:
                dict_port = {}
            account_id = get_string_from_env("AWS_ACCOUNT_ID")  # TODO this is a mess
            region_name = get_string_from_env("AWS_REGION_NAME")  # TODO this is a mess
            registry = DockerRegistryEcr(account_id, region_name)
            dict_env["IMAGE_TAG"] = image_tag

            container_name = image_tag.split(":")[0].split("/")[-1]
            stop_container(command_handler, container_name)
            remove_container(command_handler, container_name)
            await start_container(command_handler, registry, image_tag, container_name, dict_env, dict_port)
        else:
            print(f"Unknown `deploy_target_type`: {deploy_target_type}")
            sys.exit(1)


def main():

    asyncio.run(srai_deploy(get_deployment()))


if __name__ == "__main__":
    main()



import os
import sys

from srai_core.command_handler_ssh import CommandHandlerSsh
from srai_core.tools_docker import container_logs, get_client_ssh
from srai_core.tools_env import get_string_from_env


def main():
    if len(sys.argv) < 2:
        raise Exception("Please provide container name")
    container_name = sys.argv[1]
    if 2 < len(sys.argv):
        logs_count = int(sys.argv[2])
    else:
        logs_count = 100
    hostname = get_string_from_env("AWS_CNC_HOSTNAME")  # TODO these could even come from boto3
    username = "ubuntu"
    path_file_pem = "c:/key/lightsaildefaultkey-eu-central-1.pem"
    if not os.path.exists(path_file_pem):
        raise Exception(f"File {path_file_pem} does not exist")

    ssh_client = get_client_ssh(hostname, username, path_file_pem)
    command_handler = CommandHandlerSsh(ssh_client)
    # clone_repo(command_handler, "~") #TODO

    container_logs(command_handler, container_name, logs_count)


if __name__ == "__main__":
    main()



from srai_core.command_handler_base import CommandHandlerBase


class DockerRegistryBase:

    def __init__(self, registry_url: str, username: str, password: str):
        self.registry_url = registry_url
        self.username = username
        self.password = password

    def get_registry_url(self):
        return self.registry_url

    def get_username(self):
        return self.username

    def get_password(self):
        return self.password

    def get_command_login(self) -> str:
        return f"docker login --username {self.username} --password {self.password} {self.registry_url}"

    def login(self, command_handler: CommandHandlerBase) -> None:
        command = self.get_command_login()
        command_handler.execute(command)

    def image_pull(self, command_handler: CommandHandlerBase, image_tag: str) -> None:
        command = f"docker pull {self.registry_url}/{image_tag}"
        command_handler.execute(command)



from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.bytes_store_s3 import BytesStoreS3
from srai_core.store.database_base import DatabaseBase
from srai_core.store.document_store_base import DocumentStoreBase
from srai_core.tools_s3 import bucket_list, create_client_and_resource_s3


class DatabaseS3(DatabaseBase):
    def __init__(
        self,
        aws_access_key_id: str,
        aws_secret_access_key: str,
        aws_name_region: str,
        create_bucket_if_missing: bool = True,
    ):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.aws_name_region = aws_name_region
        self.create_bucket_if_missing = create_bucket_if_missing

    def list_collection_names(self) -> list[str]:
        client_s3, resource_s3 = create_client_and_resource_s3(
            self.aws_access_key_id, self.aws_secret_access_key, self.aws_name_region
        )
        return bucket_list(client_s3, resource_s3)

    def get_document_store(self, collection_name: str) -> DocumentStoreBase:
        raise NotImplementedError("Document store is not supported for S3 databases")

    def get_bytes_store(self, collection_name: str) -> BytesStoreBase:
        return BytesStoreS3(
            self.aws_access_key_id,
            self.aws_secret_access_key,
            self.aws_name_region,
            collection_name,
            create_bucket_if_missing=self.create_bucket_if_missing,
        )



from pydantic import BaseModel

LIST_CONTENT_TYPES = ["text/plain", "image/jpeg", "video/mp4", "audio/mpeg"]


class InstanceHeader(BaseModel):
    instance_id: str
    content_type: str
    content_size: int
    created_at_posix_timestamp: int
    metadata: dict



import asyncio

from srai_core.command_handler_base import CommandHandlerBase
from srai_core.command_handler_subprocess import CommandHandlerSubprocess
from srai_core.tools_docker import list_container_name


async def srai_container_list(command_handler: CommandHandlerBase):
    # hostname = "18.185.177.93"  # TODO these could even come from boto3
    # username = "ubuntu"
    # path_file_pem = "c:/key/lightsaildefaultkey-eu-central-1.pem"
    # if not os.path.exists(path_file_pem):
    #     raise Exception(f"File {path_file_pem} does not exist")

    # ssh_client = get_client_ssh(hostname, username, path_file_pem)
    list_container_name_running = list_container_name(command_handler)
    for container_name_running in list_container_name_running:
        print(container_name_running)


if __name__ == "__main__":
    command_handler = CommandHandlerSubprocess()
    asyncio.run(srai_container_list(command_handler))



from srai_core.model.docker_registry_ecr import DockerRegistryEcr
from srai_core.tools_env import get_string_from_env


def main():
    repository = DockerRegistryEcr(get_string_from_env("AWS_ACCOUNT_ID"), get_string_from_env("AWS_REGION_NAME"))

    dict_list_images = repository.image_list()
    for repo_name, list_images in dict_list_images.items():
        print(f"Images in repository: {repo_name}")
        for image_tag in list_images:
            print(f"{repo_name}:{image_tag}")


if __name__ == "__main__":
    main()



from abc import ABC, abstractmethod
from copy import copy
from typing import List

from srai_core.encoder_base import EncoderBase
from srai_core.model.encoding import Encoding
from srai_core.model.instance_header import InstanceHeader
from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.document_store_base import DocumentStoreBase


class EncoderWapper(ABC):

    def __init__(
        self,
        encoder: EncoderBase,
        instance_header_store: DocumentStoreBase,
        instance_store: BytesStoreBase,
        encoding_store: DocumentStoreBase,
    ):
        self.encoder = encoder
        self.encoder.load()
        self.instance_header_store = instance_header_store
        self.instance_store = instance_store
        self.encoding_store = encoding_store

    @property
    def encoder_id(self):
        return self.encoder.encoder_header.encoder_id

    @property
    def list_accepted_content_type(self) -> List[str]:
        return copy(self.encoder.list_accepted_content_type)

    @abstractmethod
    def encode(self, instance_id: str) -> Encoding:
        encoding_id = Encoding.create_encoding_id(
            instance_id=instance_id, encoder_id=self.encoder.encoder_header.encoder_id
        )
        document = self.encoding_store.try_load_document(encoding_id)
        if document is not None:
            return Encoding(**document)
        instance_header = InstanceHeader(**self.instance_header_store.load_document(instance_id))
        if instance_header.content_type not in self.list_accepted_content_type:
            raise ValueError(
                f"Encoder {self.encoder.encoder_header.encoder_id} is not allowed for instance {instance_id}"
            )
        instance_bytes = self.instance_store.load_bytes(instance_id)
        encoding = self.encoder.encode(instance_bytes)
        self.encoding_store.save_document(encoding_id, encoding.model_dump())
        return encoding



from abc import ABC, abstractmethod
from typing import Dict, List, Optional, Tuple


class DocumentStoreBase(ABC):
    # file store for

    def __init__(self) -> None:
        pass

    def copy_to(self, store_to: "DocumentStoreBase") -> None:
        dict_document = self.load_document_all()
        for document_id, document in dict_document.items():
            store_to.save_document(document_id, document)

    @abstractmethod
    def exists_document(self, document_id: str) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def count_document(self) -> int:
        raise NotImplementedError()

    @abstractmethod
    def load_document(self, document_id: str) -> dict:
        raise NotImplementedError()

    def try_load_document(self, bytes_id: str) -> Optional[dict]:
        if self.exists_document(bytes_id):
            return self.load_document(bytes_id)
        else:
            return None

    @abstractmethod
    def load_document_all(self) -> Dict[str, dict]:
        raise NotImplementedError()

    @abstractmethod
    def load_list_document_id(self) -> list:
        raise NotImplementedError()

    def load_document_dict_for_query(
        self,
        query: Dict[str, str],
        order_by: List[Tuple[str, bool]] = [],
        limit: int = 0,
        offset: int = 0,
    ) -> Dict[str, dict]:
        dict_document = self.load_document_all()
        dict_selected = {}

        for document_id, document in list(dict_document.items())[offset : offset + limit]:
            for key in query:
                if key not in document:
                    continue
                if document[key] == query[key]:
                    dict_selected[document_id] = document

        return dict_selected

    def load_document_for_query(self, query: Dict[str, str]) -> Optional[dict]:
        dict_document = self.load_document_dict_for_query(query, limit=1, offset=0)
        if len(dict_document) == 0:
            return None
        if len(dict_document) > 1:
            raise Exception(f"Query '{query}' returned multiple documents")
        return dict_document[list(dict_document.keys())[0]]

    @abstractmethod
    def delete_document(self, document_id: str) -> None:
        raise NotImplementedError()

    def delete_document_all(self) -> int:
        dict_document = self.load_list_document_id()
        for document_id in dict_document:
            self.delete_document(document_id)
        return len(dict_document)

    @abstractmethod
    def save_document(self, document_id: str, document: dict) -> None:
        raise NotImplementedError()



import asyncio
import sys

from srai_core.command_handler_base import CommandHandlerBase
from srai_core.script.srai_build import srai_build
from srai_core.script.srai_release_code_public import srai_release_code_public
from srai_core.tools_docker import release_docker_local_to_aws
from srai_core.tools_env import get_deployment


async def srai_release(deployment: dict) -> None:
    # No username means that the image is local

    await srai_build(deployment)
    for release_target in deployment["list_release_target"]:
        command_handler = CommandHandlerBase.from_dict(release_target["command_handler"])
        release_target_type = release_target["release_target_type"]
        if release_target_type == "release_code_public":
            await srai_release_code_public(command_handler)
        elif release_target_type == "release_docker_local_to_aws":
            await release_docker_local_to_aws(command_handler)
        else:
            print(f"Unknown release_type: {release_target_type}")
            sys.exit(1)


def main():
    asyncio.run(srai_release(get_deployment()))


if __name__ == "__main__":
    main()



from typing import Dict

from srai_core.store.document_store_base import DocumentStoreBase


class DocumentStoreCached(DocumentStoreBase):
    # file store for

    def __init__(self, inner_store: DocumentStoreBase) -> None:
        self.inner_store = inner_store
        self.dict_document = inner_store.load_document_all()

    def exists_document(self, document_id: str) -> bool:
        return document_id in self.dict_document

    def count_document(self) -> int:
        return len(self.dict_document)

    def load_document(self, document_id: str) -> dict:
        if document_id not in self.dict_document:
            if self.inner_store.exists_document(document_id):
                self.dict_document[document_id] = self.inner_store.load_document(document_id)
                return self.dict_document[document_id]
            else:
                raise ValueError(f"Document not found: {document_id}")
        else:
            return self.dict_document[document_id]

    def load_document_all(self) -> Dict[str, dict]:
        return self.dict_document.copy()

    def delete_document(self, document_id: str) -> None:
        del self.dict_document[document_id]
        self.inner_store.delete_document(document_id)

    def save_document(self, document_id: str, document: dict) -> None:
        self.dict_document[document_id] = document
        self.inner_store.save_document(document_id, document)



import hashlib
from typing import List

from pydantic import BaseModel


class Annotation(BaseModel):
    annotation_id: str
    instance_id: str
    list_encoding_id: List[str]

    created_at_posix_timestamp: int
    metadata: dict

    @staticmethod
    def create_annotation_id(decoder_id: str, list_encoder_id: List[str]):
        code = decoder_id
        for encoder_id in list_encoder_id:
            code += encoder_id
        return hashlib.sha256(code.encode()).hexdigest()



from typing import Dict

from srai_core.model.source_header import SourceHeader
from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.document_store_base import DocumentStoreBase


class SourceStore:

    def __init__(self, header_store: DocumentStoreBase, content_store: BytesStoreBase):
        self.header_store = header_store
        self.content_store = content_store

    def save_source(self, source_id: str, source_type: str, source_content: bytes, metadata: dict = {}):
        header = SourceHeader.create(
            source_id=source_id,
            source_type=source_type,
            source_content=source_content,
            metadata=metadata,
        )

        self.header_store.save_document(source_id, header.model_dump())
        self.content_store.save_bytes(source_id, source_content)

    def load_source_content(self, source_id: str) -> bytes:
        return self.content_store.load_bytes(source_id)

    def load_source_content_string(self, source_id: str, encoding="utf-8") -> str:
        return self.content_store.load_bytes(source_id).decode(encoding)

    def load_source_header(self, source_id: str) -> SourceHeader:
        return SourceHeader(**self.header_store.load_document(source_id))

    def load_source_header_all(self) -> Dict[str, SourceHeader]:
        dict_source_header = {}
        for source_id, document in self.header_store.load_document_all().items():
            dict_source_header[source_id] = SourceHeader(**document)
        return dict_source_header

    def load_list_source_id(self):
        return self.header_store.load_list_document_id()



from abc import ABC, abstractmethod
from typing import Tuple


class CommandHandlerBase(ABC):
    def __init__(self) -> None:
        pass

    @abstractmethod
    def execute(self, command: str) -> Tuple[str, str]:
        raise NotImplementedError()

    @staticmethod
    def from_dict(dict_command_handler: dict) -> "CommandHandlerBase":
        type = dict_command_handler["__Type__"]

        if type == "CommandHandlerSubprocess":
            from srai_core.command_handler_subprocess import CommandHandlerSubprocess

            return CommandHandlerSubprocess.from_dict(dict_command_handler)
        elif type == "CommandHandlerSsh":
            from srai_core.command_handler_ssh import CommandHandlerSsh

            return CommandHandlerSsh.from_dict(dict_command_handler)
        else:
            raise NotImplementedError()



from abc import ABC, abstractmethod
from typing import Type, TypeVar

from pydantic import BaseModel

from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.dir_store_base import DirStoreBase
from srai_core.store.dir_store_bytes_store import DirStoreBytesStore
from srai_core.store.document_store_base import DocumentStoreBase
from srai_core.store.object_store import ObjectStore

T = TypeVar("T", bound=BaseModel)


class DatabaseBase(ABC):
    def __init__(self, database_name: str):
        self.database_name = database_name

    def get_object_store(self, collection_name: str, model_class: Type[T]) -> ObjectStore[T]:
        return ObjectStore[model_class](self.get_document_store(collection_name), model_class)

    @abstractmethod
    def get_document_store(self, collection_name: str) -> DocumentStoreBase:
        raise NotImplementedError()

    @abstractmethod
    def get_bytes_store(self, collection_name: str) -> BytesStoreBase:
        raise NotImplementedError()

    def get_dir_store(self, collection_name: str) -> DirStoreBase:
        return DirStoreBytesStore(self.get_bytes_store(collection_name))



import asyncio
import os

from srai_core.command_handler_ssh import CommandHandlerSsh
from srai_core.tools_docker import (
    build_docker,
    clone_repository,
    get_client_ssh,
    get_project_name,
    release_docker_local_to_aws,
)
from srai_core.tools_env import get_string_from_env


async def srai_release_docker_aws_ssh():
    hostname = get_string_from_env("AWS_CNC_HOSTNAME")  # TODO these could even come from boto3
    git_token = get_string_from_env("GITHUB_TOKEN")
    username = "ubuntu"
    path_file_pem = "c:/key/lightsaildefaultkey-eu-central-1.pem"
    if not os.path.exists(path_file_pem):
        raise Exception(f"File {path_file_pem} does not exist")

    ssh_client = get_client_ssh(hostname, username, path_file_pem)
    repository_name = get_project_name()
    command_handler = CommandHandlerSsh(ssh_client)

    clone_repository(command_handler, repository_name, "~", git_token)  # TODO
    path_target = f"~/{repository_name}"

    await build_docker(command_handler, path_target)
    await release_docker_local_to_aws(command_handler)


if __name__ == "__main__":
    asyncio.run(srai_release_docker_aws_ssh())



from typing import List

from pydantic import BaseModel


class EncoderHeader(BaseModel):
    encoder_id: str
    encoder_name: str
    list_accepted_content_type: List[str]
    created_at_posix_timestamp: int
    dimension: int
    metadata: dict



from base64 import b64decode, b64encode
from typing import Dict, List

from pymongo import MongoClient

from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.document_store_mongo import DocumentStoreMongo


class BytesStoreMongo(BytesStoreBase):

    def __init__(self, client: MongoClient, database_name: str, collection_name: str):
        self.document_store_mongo = DocumentStoreMongo(client, database_name, collection_name)

    def exists_bytes(self, bytes_id: str) -> bool:
        return self.document_store_mongo.exists_document(bytes_id)

    def count_bytes(self) -> int:
        return self.document_store_mongo.count_document()

    def load_bytes(self, bytes_id: str) -> bytes:
        document = self.document_store_mongo.load_document(bytes_id)
        return b64decode(document["bytesbase64"])

    def load_list_bytes_id(self) -> List[str]:
        return self.document_store_mongo.load_list_document_id()

    def load_bytes_all(self) -> Dict[str, bytes]:
        dict_document = self.document_store_mongo.load_document_all()
        dict_bytes = {}
        for bytes_id, document in dict_document.items():
            dict_bytes[bytes_id] = b64decode(document["bytesbase64"])
        return dict_bytes

    def delete_bytes(self, bytes_id: str) -> None:
        self.document_store_mongo.delete_document(bytes_id)

    def save_bytes(self, bytes_id: str, bytes: bytes) -> None:
        document = {"bytesbase64": b64encode(bytes).decode()}
        self.document_store_mongo.save_document(bytes_id, document)



import asyncio
import json
import sys

from srai_core.command_handler_base import CommandHandlerBase
from srai_core.tools_docker import build_docker
from srai_core.tools_env import get_deployment


async def srai_build(deployment: dict) -> None:
    for build_target in deployment["list_build_target"]:
        if "build_target_type" not in build_target:
            print("`build_target_type` type not found")
            print(json.dumps(build_target, indent=4))
            sys.exit(1)

        if build_target["build_target_type"] == "build_docker":
            await build_docker(CommandHandlerBase.from_dict(build_target["command_handler"]))
        else:
            print(f"Unknown build target type: {build_target['type']}")
            sys.exit(1)


def main():
    asyncio.run(srai_build(get_deployment()))


if __name__ == "__main__":
    main()



from typing import Dict

from srai_core.store.bytes_store_base import BytesStoreBase


class BytesStoreMemory(BytesStoreBase):
    def __init__(self):
        self.dict_bytes = {}

    def save_bytes(self, bytes_id: str, bytes: bytes) -> None:
        self.dict_bytes[bytes_id] = bytes

    def load_bytes(self, bytes_id: str) -> bytes:
        return self.dict_bytes[bytes_id]

    def delete_bytes(self, bytes_id: str) -> None:
        del self.dict_bytes[bytes_id]

    def exists_bytes(self, bytes_id: str) -> bool:
        return bytes_id in self.dict_bytes

    def load_list_bytes_id(self) -> list:
        return list(self.dict_bytes.keys())

    def load_bytes_all(self) -> Dict[str, bytes]:
        return self.dict_bytes

    def count_bytes(self) -> int:
        return len(self.dict_bytes)



import hashlib

from pydantic import BaseModel

from srai_core.tools_env import get_posix_timestamp


class SourceHeader(BaseModel):
    source_id: str
    source_type: str
    created_at_posix_timestamp: int
    content_hash: str
    metadata: dict

    @staticmethod
    def create(source_id: str, source_type: str, source_content: bytes, metadata: dict):
        created_at_posix_timestamp = get_posix_timestamp()
        source_content_hash = hashlib.sha256(source_content).hexdigest()

        return SourceHeader(
            source_id=source_id,
            source_type=source_type,
            created_at_posix_timestamp=created_at_posix_timestamp,
            content_hash=source_content_hash,
            metadata=metadata,
        )



import os

from srai_core.command_handler_ssh import CommandHandlerSsh
from srai_core.tools_docker import get_client_ssh, list_container_status


def main():
    hostname = "18.185.177.93"  # TODO these could even come from boto3
    username = "ubuntu"
    path_file_pem = "c:/key/lightsaildefaultkey-eu-central-1.pem"
    if not os.path.exists(path_file_pem):
        raise Exception(f"File {path_file_pem} does not exist")

    ssh_client = get_client_ssh(hostname, username, path_file_pem)
    command_handler = CommandHandlerSsh(ssh_client)
    container_statuss = list_container_status(command_handler)
    for container_status in container_statuss:
        print(container_status["NAMES"])
        print("--" + container_status["IMAGE"])
        print("--" + container_status["STATUS"])


if __name__ == "__main__":
    main()



import datetime
import json
import os
import sys
import time
from base64 import b64decode
from typing import Any

import boto3


def get_deployment() -> dict:
    if len(sys.argv) == 2:
        path_file_deployment = sys.argv[1]
        with open(path_file_deployment) as file:
            return json.load(file)
    else:
        if os.path.isfile("deployment.json"):
            print("Found default 'deployment.json' file")
            path_file_deployment = "deployment.json"
            with open(path_file_deployment) as file:
                deployment = json.load(file)
        elif os.path.isfile("dockerfile"):
            print("Found no deployment file but a dockerfile docker release:")
            deployment = {
                "list_build_target": [
                    {
                        "command_handler": {"__Type__": "CommandHandlerSubprocess"},
                        "build_docker_type": "build_docker",
                    }
                ],
                "list_release_target": [],
                "list_deploy_target": [],
            }
        else:
            print("Found no deployment file and no dockerfile doing default realese:")
            deployment = {
                "list_build_target": [],
                "list_release_target": [
                    {
                        "command_handler": {"__Type__": "CommandHandlerSubprocess"},
                        "release_target_type": "release_code_public",
                    }
                ],
                "list_deploy_target": [],
            }
        return deployment


def get_posix_timestamp():  # TODO move to core
    return int(time.mktime(datetime.datetime.now().timetuple()))


def get_string_from_env(env_name: str) -> str:
    value = os.environ.get(env_name)
    if value is None:
        raise Exception(f"Environment variable {env_name} is not set")
    return value


def get_string_from_env_base64(env_name: str) -> str:
    value_base64 = get_string_from_env(env_name)
    return b64decode(value_base64).decode("utf-8")


def get_path_from_env(env_name: str) -> str:
    path_file = get_string_from_env(env_name)
    if not os.path.exists(path_file):
        raise Exception(f"Path {path_file} does not exist")
    return path_file


def get_client_ec2() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "ec2",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_resource_ec2() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.resource(
        "ec2",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_client_s3() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "s3",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_client_ecr() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "ecr",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_client_ecs() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "ecs",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )



from typing import Dict, List, Optional, Tuple

from pymongo.mongo_client import MongoClient

from srai_core.store.document_store_base import DocumentStoreBase


class DocumentStoreMongo(DocumentStoreBase):

    def __init__(self, client: MongoClient, database_name: str, collection_name: str):
        self.client = client
        self.db = self.client.get_database(database_name)
        self.collection = self.db.get_collection(collection_name)

    def exists_document(self, document_id) -> bool:
        return self.collection.count_documents({"_id": document_id}) > 0

    def count_document(self) -> int:
        return self.collection.count_documents({})

    def load_document(self, document_id: str) -> dict:
        return self.try_load_document(document_id, raise_if_missing=True)  # type: ignore

    def load_list_document_id(self) -> list:
        cursor_document_result = self.collection.find({})
        return [document_result["_id"] for document_result in cursor_document_result]

    def try_load_document(self, document_id: str, raise_if_missing=False) -> Optional[dict]:
        document_result = self.collection.find_one({"_id": document_id})
        if raise_if_missing and document_result is None:
            raise Exception(f"Document with id '{document_id}' not found")
        if document_result is None:
            return None
        return document_result["document"]

    def load_document_all(self) -> Dict[str, dict]:
        cursor_document_result = self.collection.find({})
        dict_document = {}
        for document_result in cursor_document_result:
            dict_document[document_result["_id"]] = document_result["document"]
        return dict_document

    def load_document_dict_for_query(
        self,
        query: Dict[str, str],
        order_by: List[Tuple[str, bool]] = [],
        limit: int = 0,
        offset: int = 0,
    ) -> Dict[str, dict]:
        query_mod = {}
        for key in query:
            query_mod["document." + key] = query[key]
        order_mod = []
        for field, asc in order_by:
            order_mod.append(("document." + field, 1 if asc else -1))

        cursor = self.collection.find(query_mod)
        # check if cursor is empty
        # if not cursor.alive:
        #     return {}
        if len(order_mod) > 0:
            cursor = cursor.sort(order_mod)
        if limit > 0:
            cursor = cursor.limit(limit)
        if offset > 0:
            cursor = cursor.skip(offset)

        dict_document = {}
        for document_result in cursor:
            dict_document[document_result["_id"]] = document_result["document"]
        return dict_document

    def save_document(self, document_id: str, document: dict, update_if_exist=True) -> None:
        if update_if_exist and self.exists_document(document_id):
            self.collection.update_one({"_id": document_id}, {"$set": {"document": document}})
        else:
            self.collection.insert_one({"_id": document_id, "document": document})

    def delete_document(self, document_id: str) -> None:
        query = {"_id": document_id}
        self.collection.delete_one(query)

    def delete_document_all(self) -> int:
        delete_result = self.collection.delete_many({})
        return delete_result.deleted_count



from abc import ABC, abstractmethod
from copy import copy
from typing import List

from srai_core.model.encoder_header import EncoderHeader
from srai_core.model.encoding import Encoding


class EncoderBase(ABC):

    def __init__(self, encoder_header: EncoderHeader):
        self.encoder_header = encoder_header

    def load(self) -> None:
        pass

    def save(self) -> None:
        pass

    @property
    def list_accepted_content_type(self) -> List[str]:
        return copy(self.encoder_header.list_accepted_content_type)

    @abstractmethod
    def encode(self, instance_bytes: bytes) -> Encoding:
        raise NotImplementedError()



import os
from io import BytesIO
from typing import List
from zipfile import ZipFile

from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.dir_store_base import DirStoreBase


class DirStoreBytesStore(DirStoreBase):
    def __init__(self, bytes_store: BytesStoreBase):
        self.bytes_store = bytes_store

    def exists_dir(self, dir_id: str) -> bool:
        return self.bytes_store.exists_bytes(dir_id)

    def count_dir(self) -> int:
        return self.bytes_store.count_bytes()

    def load_list_dir_id(self) -> List[str]:
        return list(self.bytes_store.load_bytes_all().keys())

    def delete_dir(self, dir_id: str) -> None:
        self.bytes_store.delete_bytes(dir_id)

    def save_dir(self, dir_id, path_dir_source) -> None:
        dir_bytes = DirStoreBytesStore.dir_to_bytes(path_dir_source)
        self.bytes_store.save_bytes(dir_id, dir_bytes)

    def load_dir(self, dir_id, path_dir_target) -> None:
        dir_bytes = self.bytes_store.load_bytes(dir_id)
        DirStoreBytesStore.bytes_to_dir(dir_bytes, path_dir_target)

    @staticmethod
    def dir_to_bytes(path_dir_source: str) -> bytes:
        # zip the directory into a byte string
        bytes_io = BytesIO()
        with ZipFile(bytes_io, "w") as zip_file:
            for root, dirs, files in os.walk(path_dir_source):
                for file in files:
                    file_path = os.path.join(root, file)
                    file_path_in_zip = os.path.relpath(file_path, path_dir_source)
                    zip_file.write(file_path, file_path_in_zip)
        return bytes_io.getvalue()

    @staticmethod
    def bytes_to_dir(dir_bytes: bytes, path_dir_target: str) -> None:
        # zip the directory into a byte string
        bytes_io = BytesIO(dir_bytes)
        with ZipFile(bytes_io, "r") as zip_file:
            zip_file.extractall(path_dir_target)
